from typing import Any

from pymammotion.data.model.device_limits import DeviceLimits
from pymammotion.utility.device_type import DeviceType

default_luba_config = {
    "blade_height": {"min": 20, "max": 35},
    "working_speed": {"min": 0.2, "max": 1.2},
    "path_spacing": {"min": 20, "max": 35},
    "work_area_num_max": 60,
    "display_image_type": 0,
}

default_yuka_config = {
    "blade_height": {"min": 0, "max": 0},
    "working_speed": {"min": 0.2, "max": 0.6},
    "path_spacing": {"min": 8, "max": 14},
    "work_area_num_max": 60,
    "display_image_type": 0,
}


class DeviceConfig:
    def __init__(self) -> None:
        # Dictionary to store all device configurations

        # Device mode configurations
        self.default_list: dict[str, dict[str, Any]] = {
            "a1ZU6bdGjaM": {
                "extMod": "LubaAWD1000723",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 0.4,
                "work_area_num_max": 3,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 1,
            },
            "a1nf9kRBWoH": {
                "extMod": "LubaAWD3000723",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 6,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 1,
            },
            "a1ae1QnXZGf": {
                "extMod": "LubaAWD1000743",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 3,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 1,
            },
            "a1K4Ki2L5rK": {
                "extMod": "LubaAWD5000723",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "a1jOhAYOIG8": {
                "extMod": "LubaAWD5000743LS",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "a1BmXWlsdbA": {
                "extMod": "LubaAWD5000743",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "a1JFpmAV5Ur": {
                "extMod": "Kumar-10",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "a1kweSOPylG": {
                "extMod": "LubaAWD3000723",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 6,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 1,
            },
            "a1pvCnb3PPu": {
                "extMod": "LubaAWD1000743",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 3,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 1,
            },
            "a1x0zHD3Xop": {
                "extMod": "LubaAWD5000743LS",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "a1UBFdq6nNz": {
                "extMod": "LubaAWD5000723",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "a1FbaU4Bqk5": {
                "extMod": "LubaAWD5000743",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
        }

        self.inner_list: dict[str, dict[str, Any]] = {
            "HM010060LBAWD10": {
                "extMod": "LubaAWD1000",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 3,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 1,
            },
            "HM030080LBAWD30": {
                "extMod": "LubaAWD3000",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 6,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 1,
            },
            "HM050080LBAWD50": {
                "extMod": "LubaAWD5000",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM030060LBAWD50OMNI": {
                "extMod": "LubaAWD5000",
                "blade_height_min": 30,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 1,
            },
            "HM060100LBAWD50OMNIH": {
                "extMod": "LubaAWD5000H",
                "blade_height_min": 60,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM030070LBVAWD10OMNI": {
                "extMod": "Luba2AWD1000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM060100LBVAWD10OMNIH": {
                "extMod": "Luba2AWD1000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM030070LBVAWD30OMNI": {
                "extMod": "Luba2AWD3000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 20,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM060100LBVAWD30OMNIH": {
                "extMod": "Luba2AWD3000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 20,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM030070LBVAWD50OMNI": {
                "extMod": "Luba2AWD5000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 30,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM060100LBVAWD50OMNIH": {
                "extMod": "Luba2AWD5000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 30,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM030070LBVAWD100OMNI": {
                "extMod": "Luba2AWD10000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM060100LBVAWD100OMNIH": {
                "extMod": "Luba2AWD10000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM030070LB2PAWD30OMNI": {
                "extMod": "Luba2ProAWD3000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 0.8,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM060100LB2PAWD30OMNIH": {
                "extMod": "Luba2ProAWD3000NA",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 0.8,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM030070LB2PAWD50OMNI": {
                "extMod": "Luba2ProAWD5000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM060100LB2PAWD50OMNIH": {
                "extMod": "Luba2ProAWD5000NA",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM030070LB2PAWD100OMNI": {
                "extMod": "Luba2ProAWD10000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM060100LB2PAWD100OMNIH": {
                "extMod": "Luba2ProAWD10000NA",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM020065LB2MINIAWD08OMNI": {
                "extMod": "Luba2MiniAWD800",
                "blade_height_min": 20,
                "blade_height_max": 65,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 5,
                "path_spacing_max": 20,
                "display_image_type": 0,
            },
            "HM020065LB2MINIAWD15OMNI": {
                "extMod": "Luba2MiniAWD1500",
                "blade_height_min": 20,
                "blade_height_max": 65,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 5,
                "path_spacing_max": 20,
                "display_image_type": 0,
            },
            "HM020065LB2MINIAWD15OMNILD": {
                "extMod": "Luba2MiniAWD1500Lidar",
                "blade_height_min": 20,
                "blade_height_max": 65,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 5,
                "path_spacing_max": 20,
                "display_image_type": 0,
            },
            "HM055100LB2MINIAWD08OMNIH": {
                "extMod": "Luba2MiniAWD800NA",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 5,
                "path_spacing_max": 20,
                "display_image_type": 0,
            },
            "HM055100LB2MINIAWD15OMNIH": {
                "extMod": "Luba2MiniAWD1500NA",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 5,
                "path_spacing_max": 20,
                "display_image_type": 0,
            },
            "HM055100LB2MINIAWD15OMNIHLD": {
                "extMod": "Luba2MiniAWD1500NALidar",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 5,
                "path_spacing_max": 20,
                "display_image_type": 0,
            },
            "HM030070YK06": {
                "extMod": "Yuka600",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM030100YK06H": {
                "extMod": "Yuka600NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM030070YK10": {
                "extMod": "Yuka1000",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM030100YK10H": {
                "extMod": "Yuka1000NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM030070YK15": {
                "extMod": "Yuka1500",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM030100YK15H": {
                "extMod": "Yuka1500NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM020090YK20": {
                "extMod": "Yuka2000",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM030100YK20H": {
                "extMod": "Yuka2000NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM020080YKMINI05": {
                "extMod": "YukaMini500",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "HM050090YKMINI05H": {
                "extMod": "YukaMini500NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "HM020080YKMINI08": {
                "extMod": "YukaMini800",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "HM050090YKMINI08H": {
                "extMod": "YukaMini800NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "HM020080YKMINI06": {
                "extMod": "YukaMini600",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "HM020080YKMINI07": {
                "extMod": "YukaMini700",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "HM050090YKMINI06H": {
                "extMod": "YukaMini600NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "HM050090YKMINI07H": {
                "extMod": "YukaMini700NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "HM020090YKPLUS15": {
                "extMod": "YukaPlus1500",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM020090YKPLUS20": {
                "extMod": "YukaPlus2000",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM030100YKPLUS15H": {
                "extMod": "YukaPlus1500NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM030100YKPLUS20H": {
                "extMod": "YukaPlus2000NA",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 15,
                "path_spacing_max": 30,
                "display_image_type": 0,
            },
            "HM020080MN23103": {
                "extMod": "MN231_1",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "HM020060YKMNL10": {
                "extMod": "YUKA mini 1000 LIDAR",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "HM020065LB2MINIAWDL10OMNI": {
                "extMod": "LUBA mini AWD LIDAR 1000",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "HM020065LB2MINIAWDLA20OMNI": {
                "extMod": "LUBA mini AWD 360 LIDAR 2000",
                "blade_height_min": 0,
                "blade_height_max": 0,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "100": {
                "extMod": "MN100",
                "blade_height_min": 50,
                "blade_height_max": 90,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "HM020065LB2MINIAWDL20OMNI": {
                "extMod": "LUBA mini AWD LIDAR 2000",
                "blade_height_min": 20,
                "blade_height_max": 65,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "101": {
                "extMod": "LUBA mini Vision 2000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM025070LB2PAWDL10OMNI": {
                "extMod": "LUBA 2 AWD LiDAR 1000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "61": {
                "extMod": "LUBA 2 AWD LiDAR 1000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM055100LB2PAWDL10OMNIH": {
                "extMod": "LUBA 2 AWD LiDAR 1000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "62": {
                "extMod": "LUBA 2 AWD LiDAR 1000H",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM025070LB2PAWDL30OMNI": {
                "extMod": "LUBA 2 AWD LiDAR 3000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "63": {
                "extMod": "LUBA 2 AWD LiDAR 3000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM055100LB2PAWDL30OMNIH": {
                "extMod": "LUBA 2 AWD LiDAR 3000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "64": {
                "extMod": "LUBA 2 AWD LiDAR 3000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM025070LB2PAWDL50OMNI": {
                "extMod": "LUBA 2 AWD LiDAR 5000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "65": {
                "extMod": "LUBA 2 AWD LiDAR 5000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM055100LB2PAWDL50OMNIH": {
                "extMod": "LUBA 2 AWD LiDAR 5000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "66": {
                "extMod": "LUBA 2 AWD LiDAR 5000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM025070LB2PAWDL100OMNI": {
                "extMod": "LUBA 2 AWD LiDAR 10000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "67": {
                "extMod": "LUBA 2 AWD LiDAR 10000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "HM055100LB2PAWDL100OMNIH": {
                "extMod": "LUBA 2 AWD LiDAR 10000H",
                "blade_height_min": 55,
                "blade_height_max": 100,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "68": {
                "extMod": "LUBA 2 AWD LiDAR 10000H",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "104": {
                "extMod": "LUBA 2 AWD LiDAR 5000",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 20,
                "path_spacing_max": 35,
                "display_image_type": 0,
            },
            "70": {
                "extMod": "LUBA mini AWD 360 LIDAR 2000",
                "blade_height_min": 20,
                "blade_height_max": 65,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "55": {
                "extMod": "YUKA mini 1000 LIDAR",
                "blade_height_min": 20,
                "blade_height_max": 60,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "56": {
                "extMod": "LUBA mini AWD LIDAR 2000",
                "blade_height_min": 20,
                "blade_height_max": 65,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "71": {
                "extMod": "",
                "blade_height_min": 20,
                "blade_height_max": 65,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "72": {
                "extMod": "",
                "blade_height_min": 20,
                "blade_height_max": 65,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 14,
                "display_image_type": 0,
            },
            "CM900A01": {
                "extMod": "CM900A01",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.2,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 70,
                "display_image_type": 0,
            },
            "58": {
                "extMod": "CM900A01",
                "blade_height_min": 25,
                "blade_height_max": 70,
                "working_speed_min": 0.2,
                "working_speed_max": 1.2,
                "work_area_num_max": 10,
                "path_spacing_min": 20,
                "path_spacing_max": 70,
                "display_image_type": 0,
            },
            "49": {
                "extMod": "YUKA mini Vision 500",
                "blade_height_min": 50,
                "blade_height_max": 90,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "48": {
                "extMod": "YUKA mini Vision 700",
                "blade_height_min": 50,
                "blade_height_max": 90,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "50": {
                "extMod": "YUKA mini Vision 800",
                "blade_height_min": 50,
                "blade_height_max": 90,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "37": {
                "extMod": "YUKA mini 600",
                "blade_height_min": 20,
                "blade_height_max": 60,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "38": {
                "extMod": "YUKA mini 700",
                "blade_height_min": 20,
                "blade_height_max": 60,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "39": {
                "extMod": "YUKA mini 600H",
                "blade_height_min": 50,
                "blade_height_max": 90,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
            "40": {
                "extMod": "YUKA mini 700H",
                "blade_height_min": 50,
                "blade_height_max": 90,
                "working_speed_min": 0.2,
                "working_speed_max": 1.0,
                "work_area_num_max": 60,
                "path_spacing_min": 8,
                "path_spacing_max": 12,
                "display_image_type": 0,
            },
        }

    def get_device_config(self, int_mod_or_key: str) -> dict[str, Any] | None:
        """Look up device configuration by internal model code

        Args:
            int_mod (str): Internal model code

        Returns:
            dict: Device configuration or None if not found
            :param int_mod_or_key:

        """
        if found := self.inner_list.get(int_mod_or_key) or (found := self.default_list.get(int_mod_or_key)):
            return found
        return None

    def get_external_model(self, int_mod: str) -> str | None:
        """Get external model name for given internal model code

        Args:
            int_mod (str): Internal model code

        Returns:
            str: External model name or None if not found

        """
        config = self.get_device_config(int_mod)
        return config.get("extMod") if config else None

    def get_working_parameters(self, int_mod_or_key: str) -> DeviceLimits | None:
        """Get working parameters for given internal model code

        Args:
            int_mod (str): Internal model code

        Returns:
            dict: Working parameters or None if not found
            :param int_mod_or_key:

        """
        if config := self.get_device_config(int_mod_or_key):
            return DeviceLimits.from_dict(
                {
                    "blade_height": {"min": config["blade_height_min"], "max": config["blade_height_max"]},
                    "working_speed": {"min": config["working_speed_min"], "max": config["working_speed_max"]},
                    "path_spacing": {"min": config["path_spacing_min"], "max": config["path_spacing_max"]},
                    "work_area_num_max": config["work_area_num_max"],
                    "display_image_type": config["display_image_type"],
                }
            )
        return None

    @staticmethod
    def get_best_default(product_key: str) -> DeviceLimits:
        """Basic fallback if device is offline."""

        if DeviceType.contain_luba_product_key(product_key):
            return DeviceLimits.from_dict(default_luba_config)
        if DeviceType.contain_luba_2_product_key(product_key):
            return DeviceLimits.from_dict(default_luba_config)

        return DeviceLimits.from_dict(default_yuka_config)


# # Usage example:
# def main():
#     device_config = DeviceConfig()
#
#     # Look up a specific device
#     model_code = "HM010060LBAWD10"
#
#     # Get full configuration
#     config = device_config.get_device_config(model_code)
#     print(f"Full configuration for {model_code}:")
#     print(config)
#
#     # Get external model name
#     ext_model = device_config.get_external_model(model_code)
#     print(f"\nExternal model name: {ext_model}")
#
#     # Get working parameters
#     params = device_config.get_working_parameters(model_code)
#     print(f"\nWorking parameters:")
#     print(params)



import asyncio
from collections.abc import Awaitable, Callable
import json
import logging
import ssl
from typing import Any
from urllib.parse import urlparse

import paho.mqtt.client as mqtt
from paho.mqtt.properties import Properties
from paho.mqtt.reasoncodes import ReasonCode

from pymammotion import MammotionHTTP
from pymammotion.http.model.http import DeviceRecord, MQTTConnection, Response, UnauthorizedException
from pymammotion.utility.datatype_converter import DatatypeConverter

logger = logging.getLogger(__name__)


class MammotionMQTT:
    """Mammotion MQTT Client."""

    converter = DatatypeConverter()

    def __init__(
        self, mqtt_connection: MQTTConnection, mammotion_http: MammotionHTTP, records: list[DeviceRecord]
    ) -> None:
        self.on_connected: Callable[[], Awaitable[None]] | None = None
        self.on_ready: Callable[[], Awaitable[None]] | None = None
        self.on_error: Callable[[str], Awaitable[None]] | None = None
        self.on_disconnected: Callable[[], Awaitable[None]] | None = None
        self.on_message: Callable[[str, bytes, str], Awaitable[None]] | None = None
        self.loop = asyncio.get_running_loop()
        self.mammotion_http = mammotion_http
        self.mqtt_connection = mqtt_connection
        self.client = self.build(mqtt_connection)

        self.records = records

        # wire callbacks from the service object if present
        self.client.on_connect = self._on_connect
        self.client.on_message = self._on_message
        self.client.on_disconnect = self._on_disconnect
        # client.on_subscribe = getattr(mqtt_service_obj, "on_subscribe", None)
        # client.on_publish = getattr(mqtt_service_obj, "on_publish", None)

    def __del__(self) -> None:
        if self.client.is_connected():
            for record in self.records:
                self.unsubscribe_all(record.product_key, record.device_name)
            self.client.disconnect()

    def connect_async(self) -> None:
        """Connect async to MQTT Server."""
        if not self.client.is_connected():
            logger.info("Connecting...")
            self.client.connect_async(host=self.client.host, port=self.client.port, keepalive=self.client.keepalive)
            self.client.loop_start()

    def disconnect(self) -> None:
        """Disconnect from MQTT Server."""
        logger.info("Disconnecting...")
        self.client.disconnect()

    @staticmethod
    def build(mqtt_connection: MQTTConnection, keepalive: int = 60, timeout: int = 30) -> mqtt.Client:
        """get_jwt_response: object with attributes .client_id, .username, .jwt (password), .host (e.g. 'mqtts://broker:8883' or 'broker:1883' or 'broker').
        mqtt_service_obj: object that exposes callback methods (on_connect, on_message, on_disconnect, etc.)
        Returns: (client, connected_bool, rc)
        """
        host = mqtt_connection.host
        # Ensure urlparse can parse plain hosts
        parsed = urlparse(host if "://" in host else "tcp://" + host)
        scheme = parsed.scheme
        hostname = parsed.hostname
        port = parsed.port

        # decide TLS/ssl and default port
        use_ssl = scheme in ("mqtts", "ssl")
        if port is None:
            port = 8883 if use_ssl else 1883

        client = mqtt.Client(
            callback_api_version=mqtt.CallbackAPIVersion.VERSION2,
            client_id=mqtt_connection.client_id,
            clean_session=True,
            protocol=mqtt.MQTTv311,
        )

        client.username_pw_set(mqtt_connection.username, mqtt_connection.jwt)

        if use_ssl:
            # use system default CA certs; adjust tls_set() params if custom CA/client certs required
            client.tls_set(cert_reqs=ssl.CERT_REQUIRED)
            client.tls_insecure_set(False)

        # automatic reconnect backoff
        client.reconnect_delay_set(min_delay=1, max_delay=120)

        # connect (synchronous connect attempt) and start background loop
        if hostname:
            client.host = hostname
            client.port = port
            client.keepalive = keepalive

        return client

    def _on_message(self, _client: mqtt.Client, _userdata: Any, message: mqtt.MQTTMessage) -> None:
        """Is called when message is received."""
        logger.debug("Message on topic %s", message.topic)
        logger.debug(message)

        if self.on_message is not None:
            iot_id = None
            # Parse the topic path to get product_key and device_name
            topic_parts = message.topic.split("/")
            if len(topic_parts) >= 4:
                product_key = topic_parts[2]
                device_name = topic_parts[3]

                # Filter records to find matching device
                filtered_records = [
                    record
                    for record in self.records
                    if record.product_key == product_key and record.device_name == device_name
                ]

                if filtered_records:
                    iot_id = filtered_records[0].iot_id
                    payload = json.loads(message.payload.decode("utf-8"))
                    payload["iot_id"] = iot_id
                    payload["product_key"] = product_key
                    payload["device_name"] = device_name
                    message.payload = json.dumps(payload).encode("utf-8")

            if iot_id:
                future = asyncio.run_coroutine_threadsafe(
                    self.on_message(message.topic, message.payload, iot_id), self.loop
                )
                asyncio.wrap_future(future, loop=self.loop)

    def _on_connect(
        self,
        _client: mqtt.Client,
        user_data: Any,
        session_flag: mqtt.ConnectFlags,
        rc: ReasonCode,
        properties: Properties | None,
    ) -> None:
        """Handle connection event and execute callback if set."""
        self.is_connected = True
        for record in self.records:
            self.subscribe_all(record.product_key, record.device_name)
        if self.on_connected is not None:
            future = asyncio.run_coroutine_threadsafe(self.on_connected(), self.loop)
            asyncio.wrap_future(future, loop=self.loop)

        if self.on_ready:
            self.is_ready = True
            future = asyncio.run_coroutine_threadsafe(self.on_ready(), self.loop)
            asyncio.wrap_future(future, loop=self.loop)

        logger.debug("on_connect, session_flag:%s, rc:%s", session_flag, rc)

    def _on_disconnect(
        self,
        _client: mqtt.Client,
        user_data: Any | None,
        disconnect_flags: mqtt.DisconnectFlags,
        rc: ReasonCode,
        properties: Properties | None,
        **kwargs: Any,
    ) -> None:
        """Handle disconnection event and execute callback if set."""
        self.is_connected = False
        if self.on_disconnected is not None:
            for record in self.records:
                self.unsubscribe_all(record.product_key, record.device_name)
            future = asyncio.run_coroutine_threadsafe(self.on_disconnected(), self.loop)
            asyncio.wrap_future(future, loop=self.loop)

        logger.debug("on_disconnect, rc:%s", rc)

    def subscribe_all(self, product_key: str, device_name: str) -> None:
        """Subscribe to all topics for the given device."""

        # "/sys/" + this.$productKey + "/" + this.$deviceName + "/thing/event/+/post"
        # "/sys/proto/" + this.$productKey + "/" + this.$deviceName + "/thing/event/+/post"
        # "/sys/" + this.$productKey + "/" + this.$deviceName + "/app/down/thing/status"
        self.client.subscribe(f"/sys/{product_key}/{device_name}/app/down/thing/status")
        self.client.subscribe(f"/sys/{product_key}/{device_name}/thing/event/+/post")
        self.client.subscribe(f"/sys/proto/{product_key}/{device_name}/thing/event/+/post")

    def unsubscribe_all(self, product_key: str, device_name: str) -> None:
        """Unsubscribe from all topics for the given device."""
        self.client.unsubscribe(f"/sys/{product_key}/{device_name}/app/down/thing/status")
        self.client.unsubscribe(f"/sys/{product_key}/{device_name}/thing/event/+/post")
        self.client.unsubscribe(f"/sys/proto/{product_key}/{device_name}/thing/event/+/post")

    async def send_cloud_command(self, iot_id: str, command: bytes) -> str:
        """Send command to cloud."""
        res: Response[dict] = await self.mammotion_http.mqtt_invoke(
            self.converter.printBase64Binary(command), "", iot_id
        )

        logger.debug("send_cloud_command: %s", res)

        if res.code == 401:
            raise UnauthorizedException(res.msg)

        if res.code != 0:
            raise Exception(f"Error sending cloud command: {res.msg}, {iot_id}")

        return str(res.data.get("result"))



"""Thin api layer between home assistant and pymammotion."""

import asyncio
from datetime import datetime, timedelta
from logging import getLogger
from typing import Any

from aiohttp import ClientSession

from pymammotion.aliyun.cloud_gateway import (
    EXPIRED_CREDENTIAL_EXCEPTIONS,
    DeviceOfflineException,
    FailedRequestException,
    GatewayTimeoutException,
    NoConnectionException,
)
from pymammotion.data.model import GenerateRouteInformation
from pymammotion.data.model.device import MowingDevice
from pymammotion.data.model.device_config import OperationSettings, create_path_order
from pymammotion.data.model.device_limits import DeviceLimits
from pymammotion.mammotion.devices import MammotionMowerDeviceManager
from pymammotion.mammotion.devices.mammotion import Mammotion
from pymammotion.proto import RptAct, RptInfoType
from pymammotion.utility.device_config import DeviceConfig
from pymammotion.utility.device_type import DeviceType

logger = getLogger(__name__)


class HomeAssistantMowerApi:
    """API for interacting with Mammotion Mowers for Home Assistant."""

    def __init__(self, session: ClientSession | None = None) -> None:
        self._device_config = DeviceConfig()
        self._plan_lock = asyncio.Lock()
        self.update_failures = 0
        self._mammotion = Mammotion(session)
        self._map_lock = asyncio.Lock()
        self._last_call_times: dict[str, datetime] = {}
        self._call_intervals = {
            "check_maps": timedelta(minutes=5),
            "read_plan": timedelta(minutes=30),
            "read_settings": timedelta(minutes=5),
            "get_errors": timedelta(minutes=1),
            "get_report_cfg": timedelta(seconds=5),
            "get_maintenance": timedelta(minutes=30),
            "device_version_upgrade": timedelta(hours=24),
            "device_info": timedelta(hours=24),
        }

    @property
    def mammotion(self) -> Mammotion:
        return self._mammotion

    def _should_call_api(self, api_name: str, device: MowingDevice | None = None) -> bool:
        """Check if API should be called based on time or criteria."""
        # Time-based check
        if api_name not in self._last_call_times:
            return True

        last_call = self._last_call_times[api_name]
        interval = self._call_intervals.get(api_name, timedelta(seconds=10))

        # Criteria-based checks
        if api_name == "check_maps" and device:
            # Call immediately if map data is incomplete
            if len(device.map.area) == 0 or device.map.missing_hashlist():
                return True

        return datetime.now() - last_call >= interval

    def _mark_api_called(self, api_name: str) -> None:
        """Mark an API as called with the current timestamp."""
        self._last_call_times[api_name] = datetime.now()

    def device_limits(self, device_name: str) -> DeviceLimits:
        device = self._mammotion.get_device_by_name(device_name)
        return self._device_config.get_best_default(device.state.mower_state.product_key)

    async def update(self, device_name: str) -> MowingDevice:
        device = self._mammotion.get_device_by_name(device_name)

        if device.has_queued_commands():
            return device.state

        if self._map_lock.locked():
            # if maps is not complete kick off the map sync process again
            if len(device.state.map.missing_hashlist()) > 0:
                await self._mammotion.start_map_sync(device_name)
                return device.state
            # if maps complete
            else:
                self._map_lock.release()

        # Check maps periodically
        if self._should_call_api("check_maps") and not self._map_lock.locked():
            await self._map_lock.acquire()
            await self.mammotion.start_map_sync(device_name)
            self._mark_api_called("check_maps")
            return device.state

        if self._should_call_api("read_plan"):
            if len(device.state.map.plan) == 0 or list(device.state.map.plan.values())[0].total_plan_num != len(
                device.state.map.plan
            ):
                await self.async_send_command(device_name, "read_plan", sub_cmd=2, plan_index=0)
                self._mark_api_called("read_plan")
                return device.state

        # Read settings less frequently
        if self._should_call_api("read_settings"):
            # await device.async_read_settings()
            self._mark_api_called("read_settings")

        # Check for errors periodically
        if self._should_call_api("get_errors"):
            await self.async_send_command(device_name, "get_error_code")
            await self.async_send_command(device_name, "get_error_timestamp")
            self._mark_api_called("get_errors")

        if self._should_call_api("get_report_cfg"):
            await self.async_send_command(device_name, "get_report_cfg")
            self._mark_api_called("get_report_cfg")

        if self._should_call_api("get_maintenance"):
            await self.async_send_command(device_name, "get_maintenance")
            self._mark_api_called("get_maintenance")

        if self._should_call_api("device_version_upgrade"):
            await self.async_check_firmware_version(device_name)
            self._mark_api_called("device_version_upgrade")

        if self._should_call_api("device_info"):
            await self.async_device_info(device_name)

        return device.state

    async def async_send_command(self, device_name: str, command: str, **kwargs: Any) -> bool | None:
        """Send command."""
        device = self._mammotion.get_device_by_name(device_name)

        try:
            # TODO check preference
            if device.cloud:
                return await self.async_send_cloud_command(device, command, **kwargs)
            elif device.ble:
                return await self.async_send_bluetooth_command(device, command, **kwargs)
        except (DeviceOfflineException, NoConnectionException) as ex:
            """Device is offline try bluetooth if we have it."""
            logger.error(f"Device offline: {ex.iot_id}")
            if ble := device.ble:
                # if we don't do this, it will stay connected and no longer update over Wi-Fi
                ble.set_disconnect_strategy(disconnect=True)
                await ble.queue_command(command, **kwargs)

                return True
            raise DeviceOfflineException(ex.args[0], device.iot_id)
        return False

    async def async_send_cloud_command(
        self, device: MammotionMowerDeviceManager, key: str, **kwargs: Any
    ) -> bool | None:
        """Send command."""
        if cloud := device.cloud:
            if not device.state.online:
                return False

            try:
                await cloud.command(key, **kwargs)
                self.update_failures = 0
                return True
            except FailedRequestException:
                self.update_failures += 1
                if self.update_failures < 5:
                    await cloud.command(key, **kwargs)
                    return True
                return False
            except EXPIRED_CREDENTIAL_EXCEPTIONS:
                self.update_failures += 1
                await self._mammotion.refresh_login(device.mammotion_http.account)
                # TODO tell home assistant the credentials have changed
                if self.update_failures < 5:
                    await cloud.command(key, **kwargs)
                    return True
                return False
            except GatewayTimeoutException as ex:
                logger.error(f"Gateway timeout exception: {ex.iot_id}")
                self.update_failures = 0
                return False
            except (DeviceOfflineException, NoConnectionException) as ex:
                """Device is offline try bluetooth if we have it."""
                logger.error(f"Device offline: {ex.iot_id}")
        return False

    @staticmethod
    async def async_send_bluetooth_command(device: MammotionMowerDeviceManager, key: str, **kwargs: Any) -> bool | None:
        """Send command."""
        if ble := device.ble:
            await ble.command(key, **kwargs)

            return True
        raise DeviceOfflineException("bluetooth command failed", device.iot_id)

    async def set_scheduled_updates(self, device_name: str, enabled: bool) -> None:
        device = self.mammotion.get_device_by_name(device_name)
        device.state.enabled = enabled
        if device.state.enabled:
            self.update_failures = 0
            if not device.state.online:
                device.state.online = True
            if device.cloud and device.cloud.stopped:
                await device.cloud.start()
        else:
            if device.cloud:
                await device.cloud.stop()
                if device.cloud.mqtt.is_connected():
                    device.cloud.mqtt.disconnect()
            if device.ble:
                await device.ble.stop()

    def is_online(self, device_name: str) -> bool:
        if device := self.mammotion.get_device_by_name(device_name):
            if ble := device.ble:
                return device.state.online or ble is not None and ble.client.is_connected
        return False

    async def update_firmware(self, device_name: str, version: str) -> None:
        """Update firmware."""
        device = self.mammotion.get_device_by_name(device_name)
        await device.mammotion_http.start_ota_upgrade(device.iot_id, version)

    async def async_start_stop_blades(self, device_name: str, start_stop: bool, blade_height: int = 60) -> None:
        """Start stop blades."""
        if DeviceType.is_luba1(device_name):
            if start_stop:
                await self.async_send_command(device_name, "set_blade_control", on_off=1)
            else:
                await self.async_send_command(device_name, "set_blade_control", on_off=0)
        elif start_stop:
            if DeviceType.is_yuka(device_name) or DeviceType.is_yuka_mini(device_name):
                blade_height = 0

            await self.async_send_command(
                device_name,
                "operate_on_device",
                main_ctrl=1,
                cut_knife_ctrl=1,
                cut_knife_height=blade_height,
                max_run_speed=1.2,
            )
        else:
            await self.async_send_command(
                device_name,
                "operate_on_device",
                main_ctrl=0,
                cut_knife_ctrl=0,
                cut_knife_height=blade_height,
                max_run_speed=1.2,
            )

    async def async_set_rain_detection(self, device_name: str, on_off: bool) -> None:
        """Set rain detection."""
        await self.async_send_command(device_name, "read_write_device", rw_id=3, context=int(on_off), rw=1)

    async def async_read_rain_detection(self, device_name: str) -> None:
        """Set rain detection."""
        await self.async_send_command(device_name, "read_write_device", rw_id=3, context=1, rw=0)

    async def async_set_sidelight(self, device_name: str, on_off: int) -> None:
        """Set Sidelight."""
        await self.async_send_command(device_name, "read_and_set_sidelight", is_sidelight=bool(on_off), operate=0)
        await self.async_read_sidelight(device_name)

    async def async_read_sidelight(self, device_name: str) -> None:
        """Set Sidelight."""
        await self.async_send_command(device_name, "read_and_set_sidelight", is_sidelight=False, operate=1)

    async def async_set_manual_light(self, device_name: str, manual_ctrl: bool) -> None:
        """Set manual night light."""
        await self.async_send_command(device_name, "set_car_manual_light", manual_ctrl=manual_ctrl)
        await self.async_send_command(device_name, "get_car_light", ids=1126)

    async def async_set_night_light(self, device_name: str, night_light: bool) -> None:
        """Set night light."""
        await self.async_send_command(device_name, "set_car_light", on_off=night_light)
        await self.async_send_command(device_name, "get_car_light", ids=1123)

    async def async_set_traversal_mode(self, device_name: str, context: int) -> None:
        """Set traversal mode."""
        await self.async_send_command(device_name, "traverse_mode", context=context)

    async def async_set_turning_mode(self, device_name: str, context: int) -> None:
        """Set turning mode."""
        await self.async_send_command(device_name, "turning_mode", context=context)

    async def async_blade_height(self, device_name: str, height: int) -> int:
        """Set blade height."""
        await self.async_send_command(device_name, "set_blade_height", height=height)
        return height

    async def async_set_cutter_speed(self, device_name: str, mode: int) -> None:
        """Set cutter speed."""
        await self.async_send_command(device_name, "set_cutter_mode", cutter_mode=mode)

    async def async_set_speed(self, device_name: str, speed: float) -> None:
        """Set working speed."""
        await self.async_send_command(device_name, "set_speed", speed=speed)

    async def async_leave_dock(self, device_name: str) -> None:
        """Leave dock."""
        await self.send_command_and_update(device_name, "leave_dock")

    async def async_cancel_task(self, device_name: str) -> None:
        """Cancel task."""
        await self.send_command_and_update(device_name, "cancel_job")

    async def async_move_forward(self, device_name: str, speed: float) -> None:
        """Move forward."""
        device = self.mammotion.get_device_by_name(device_name)
        await self.async_send_bluetooth_command(device, "move_forward", linear=speed)

    async def async_move_left(self, device_name: str, speed: float) -> None:
        """Move left."""
        device = self.mammotion.get_device_by_name(device_name)
        await self.async_send_bluetooth_command(device, "move_left", angular=speed)

    async def async_move_right(self, device_name: str, speed: float) -> None:
        """Move right."""
        device = self.mammotion.get_device_by_name(device_name)
        await self.async_send_bluetooth_command(device, "move_right", angular=speed)

    async def async_move_back(self, device_name: str, speed: float) -> None:
        """Move back."""
        device = self.mammotion.get_device_by_name(device_name)
        await self.async_send_bluetooth_command(device, "move_back", linear=speed)

    async def async_rtk_dock_location(self, device_name: str) -> None:
        """RTK and dock location."""
        await self.async_send_command(device_name, "read_write_device", rw_id=5, rw=1, context=1)

    async def async_get_area_list(self, device_name: str, iot_id: str) -> None:
        """Mowing area List."""
        await self.async_send_command(device_name, "get_area_name_list", device_id=iot_id)

    async def async_relocate_charging_station(self, device_name: str) -> None:
        """Reset charging station."""
        await self.async_send_command(device_name, "delete_charge_point")
        # fetch charging location?
        """
        nav {
          todev_get_commondata {
            pver: 1
            subCmd: 2
            action: 6
            type: 5
            totalFrame: 1
            currentFrame: 1
          }
        }
        """

    async def async_set_non_work_hours(self, device_name: str, start_time: str, end_time: str) -> None:
        """Set non work hours l1?."""
        device = self._mammotion.get_device_by_name(device_name)
        await self.async_send_command(
            device_name,
            "set_plan_unable_time",
            sub_cmd=device.state.non_work_hours.sub_cmd,
            device_id=device.iot_id,
            unable_end_time=end_time,
            unable_start_time=start_time,
        )

    async def async_set_job_dnd(self, device_name: str, start_time: str, end_time: str) -> None:
        """Set non work hours."""
        await self.async_send_command(
            device_name,
            "job_do_not_disturb",
            sub_cmd=1,
            trigger=1,
            unable_end_time=end_time,
            unable_start_time=start_time,
        )

    async def async_del_job_dnd(self, device_name: str) -> None:
        """Del non work hours."""
        await self.async_send_command(device_name, "job_do_not_disturb", sub_cmd=1, trigger=0)

    async def send_command_and_update(self, device_name: str, command_str: str, **kwargs: Any) -> None:
        """Send command and update."""
        await self.async_send_command(device_name, command_str, **kwargs)
        await self.async_request_iot_sync(device_name)

    async def async_request_iot_sync(self, device_name: str, stop: bool = False) -> None:
        """Sync specific info from device."""
        await self.async_send_command(
            device_name,
            "request_iot_sys",
            rpt_act=RptAct.RPT_STOP if stop else RptAct.RPT_START,
            rpt_info_type=[
                RptInfoType.RIT_DEV_STA,
                RptInfoType.RIT_DEV_LOCAL,
                RptInfoType.RIT_WORK,
                RptInfoType.RIT_MAINTAIN,
                RptInfoType.RIT_BASESTATION_INFO,
                RptInfoType.RIT_VIO,
            ],
            timeout=10000,
            period=3000,
            no_change_period=4000,
            count=0,
        )

    def generate_route_information(
        self, device_name: str, operation_settings: OperationSettings
    ) -> GenerateRouteInformation:
        """Generate route information."""
        device = self.mammotion.get_device_by_name(device_name)
        if device.state.report_data.dev:
            dev = device.state.report_data.dev
            if dev.collector_status.collector_installation_status == 0:
                operation_settings.is_dump = False

        if DeviceType.is_yuka(device_name):
            operation_settings.blade_height = -10

        route_information = GenerateRouteInformation(
            one_hashs=list(operation_settings.areas),
            rain_tactics=operation_settings.rain_tactics,
            speed=operation_settings.speed,
            ultra_wave=operation_settings.ultra_wave,  # touch no touch etc
            toward=operation_settings.toward,  # is just angle (route angle)
            toward_included_angle=operation_settings.toward_included_angle  # demond_angle
            if operation_settings.channel_mode == 1
            else 0,  # crossing angle relative to grid
            toward_mode=operation_settings.toward_mode,
            blade_height=operation_settings.blade_height,
            channel_mode=operation_settings.channel_mode,  # single, double, segment or none (route mode)
            channel_width=operation_settings.channel_width,  # path space
            job_mode=operation_settings.job_mode,  # taskMode grid or border first
            edge_mode=operation_settings.mowing_laps,  # perimeter/mowing laps
            path_order=create_path_order(operation_settings, device_name),
            obstacle_laps=operation_settings.obstacle_laps,
        )

        if DeviceType.is_luba1(device_name):
            route_information.toward_mode = 0
            route_information.toward_included_angle = 0
        return route_information

    async def async_plan_route(self, device_name: str, operation_settings: OperationSettings) -> bool | None:
        """Plan mow."""
        route_information = self.generate_route_information(device_name, operation_settings)

        # not sure if this is artificial limit
        # if (
        #     DeviceType.is_mini_or_x_series(device_name)
        #     and route_information.toward_mode == 0
        # ):
        #     route_information.toward = 0

        return await self.async_send_command(
            device_name, "generate_route_information", generate_route_information=route_information
        )

    async def async_modify_plan_route(self, device_name: str, operation_settings: OperationSettings) -> bool | None:
        """Modify plan mow."""
        device = self.mammotion.get_device_by_name(device_name)

        if work := device.state.work:
            operation_settings.areas = set(work.zone_hashs)
            operation_settings.toward = work.toward
            operation_settings.toward_mode = work.toward_mode
            operation_settings.toward_included_angle = work.toward_included_angle
            operation_settings.mowing_laps = work.edge_mode
            operation_settings.job_mode = work.job_mode
            operation_settings.job_id = work.job_id
            operation_settings.job_version = work.job_ver

        route_information = self.generate_route_information(device_name, operation_settings)
        if route_information.toward_mode == 0:
            route_information.toward = 0

        return await self.async_send_command(
            device_name, "modify_route_information", generate_route_information=route_information
        )

    async def start_task(self, device_name: str, plan_id: str) -> None:
        """Start task."""
        await self.async_send_command(device_name, "single_schedule", plan_id=plan_id)

    async def clear_update_failures(self, device_name: str) -> None:
        """Clear update failures."""
        self.update_failures = 0
        device = self.mammotion.get_device_by_name(device_name)
        if not device.state.online:
            device.state.online = True
        if cloud := device.cloud:
            if cloud.stopped:
                await cloud.start()

    async def async_check_firmware_version(self, device_name: str) -> None:
        """Checks firmware version."""
        device = self.mammotion.get_device_by_name(device_name)
        ota_info = await device.mammotion_http.get_device_ota_firmware([device.iot_id])
        logger.debug("OTA info: %s", ota_info.data)
        if check_versions := ota_info.data:
            for check_version in check_versions:
                if check_version.device_id == device.iot_id:
                    device.state.update_check = check_version

    async def async_device_info(self, device_name) -> None:
        """Get device info."""
        command_list = [
            "get_device_version_main",
            "get_device_version_info",
            "get_device_base_info",
            "get_device_product_model",
        ]
        for command in command_list:
            await self.async_send_command(device_name, command)



from dataclasses import dataclass
from typing import Annotated

from mashumaro.config import BaseConfig
from mashumaro.mixins.orjson import DataClassORJSONMixin
from mashumaro.types import Alias


@dataclass
class FirmwareInfo(DataClassORJSONMixin):
    t: str
    c: str
    v: str


@dataclass
class DeviceVersionInfo(DataClassORJSONMixin):
    dev_ver: Annotated[str, Alias("devVer")]
    whole: int
    fw_info: Annotated[list[FirmwareInfo], Alias("fwInfo")]

    class Config(BaseConfig):
        allow_deserialization_not_by_alias = True


@dataclass
class Coordinate(DataClassORJSONMixin):
    lon: float
    lat: float


@dataclass
class InternalNavigation(DataClassORJSONMixin):
    nav: Annotated[str, Alias("NAV")]
    pau: Annotated[str, Alias("Pau")]
    r_pau: Annotated[str, Alias("rPau")]
    mcu: Annotated[str, Alias("MCU")]
    app: Annotated[str, Alias("APP")]
    w_slp: Annotated[str, Alias("wSlp")]
    i_slp: Annotated[str, Alias("iSlp")]

    class Config(BaseConfig):
        allow_deserialization_not_by_alias = True


@dataclass
class BandwidthTraffic(DataClassORJSONMixin):
    iot: Annotated[str, Alias("IoT")]
    roi: Annotated[str, Alias("RoI")]
    fpv: Annotated[str, Alias("FPV")]
    inav: InternalNavigation

    class Config(BaseConfig):
        allow_deserialization_not_by_alias = True


@dataclass
class TrafficPeriod(DataClassORJSONMixin):
    r: str
    t: str
    s: str


@dataclass
class TrafficData(DataClassORJSONMixin):
    upt: str
    hour: Annotated[dict[str, TrafficPeriod], Alias("Hour")]
    day: Annotated[dict[str, TrafficPeriod], Alias("Day")]
    mon: Annotated[dict[str, TrafficPeriod], Alias("Mon")]

    class Config(BaseConfig):
        allow_deserialization_not_by_alias = True


@dataclass
class NetworkInfo(DataClassORJSONMixin):
    ssid: str
    ip: str
    wifi_sta_mac: str
    wifi_rssi: int
    wifi_available: int
    bt_mac: str
    mnet_model: str
    imei: str
    fw_ver: str
    sim: str
    imsi: str
    iccid: str
    sim_source: str
    mnet_rssi: int
    signal: int
    mnet_link: int
    mnet_option: str
    mnet_ip: str
    mnet_reg: str
    mnet_rsrp: str
    mnet_snr: str
    mnet_enable: int
    apn_num: int
    apn_info: str
    apn_cid: int
    used_net: int
    hub_reset: int
    mnet_dis: int
    airplane_times: int
    lsusb_num: int
    b_tra: Annotated[BandwidthTraffic, Alias("bTra")]
    bw_tra: Annotated[BandwidthTraffic, Alias("bwTra")]
    mnet_rx: str
    mnet_tx: str
    m_tra: Annotated[TrafficData, Alias("mTra")]
    mnet_uniot: int
    mnet_un_getiot: int
    ssh_flag: str
    mileage: str
    work_time: str
    wt_sec: int
    bat_cycles: str


@dataclass
class DeviceOtherInfo(DataClassORJSONMixin):
    soc_up_time: Annotated[int, Alias("socUpTime")]
    mcu_up_time: Annotated[int, Alias("mcuUpTime")]
    soc_loads: Annotated[str, Alias("socLoads")]
    soc_mem_free: Annotated[int, Alias("socMemFree")]
    soc_mem_total: Annotated[int, Alias("socMemTotal")]
    soc_mmc_life_time: Annotated[int, Alias("socMmcLifeTime")]
    usb_dis_cnt: Annotated[int, Alias("usbDisCnt")]
    soc_pstore: Annotated[int, Alias("socPstore")]
    soc_coredump: Annotated[int, Alias("socCoredump")]
    soc_tmp: Annotated[int, Alias("socTmp")]
    mc_mcu: Annotated[str, Alias("mcMcu")]
    tilt_degree: str
    i_msg_free: Annotated[int, Alias("iMsgFree")]
    i_msg_limit: Annotated[int, Alias("iMsgLimit")]
    i_msg_raw: Annotated[int, Alias("iMsgRaw")]
    i_msg_prop: Annotated[int, Alias("iMsgprop")]
    i_msg_serv: Annotated[int, Alias("iMsgServ")]
    i_msg_info: Annotated[int, Alias("iMsgInfo")]
    i_msg_warn: Annotated[int, Alias("iMsgWarn")]
    i_msg_fault: Annotated[int, Alias("iMsgFault")]
    i_msg_ota_stage: Annotated[int, Alias("iMsgOtaStage")]
    i_msg_protobuf: Annotated[int, Alias("iMsgProtobuf")]
    i_msg_notify: Annotated[int, Alias("iMsgNotify")]
    i_msg_log_prog: Annotated[int, Alias("iMsgLogProg")]
    i_msg_biz_req: Annotated[int, Alias("iMsgBizReq")]
    i_msg_cfg_req: Annotated[int, Alias("iMsgCfgReq")]
    i_msg_voice: Annotated[int, Alias("iMsgVoice")]
    i_msg_warn_code: Annotated[int, Alias("iMsgWarnCode")]
    pb_net: Annotated[int, Alias("pbNet")]
    pb_sys: Annotated[int, Alias("pbSys")]
    pb_nav: Annotated[int, Alias("pbNav")]
    pb_local: Annotated[int, Alias("pbLocal")]
    pb_plan: Annotated[int, Alias("pbPlan")]
    pb_e_drv: Annotated[int, Alias("pbEDrv")]
    pb_e_sys: Annotated[int, Alias("pbESys")]
    pb_midware: Annotated[int, Alias("pbMidware")]
    pb_ota: Annotated[int, Alias("pbOta")]
    pb_appl: Annotated[int, Alias("pbAppl")]
    pb_mul: Annotated[int, Alias("pbMul")]
    pb_other: Annotated[int, Alias("pbOther")]
    lora_connect: Annotated[int, Alias("loraConnect")]
    base_status: Annotated[int, Alias("Basestatus")]
    mqtt_rtk_switch: int
    mqtt_rtk_channel: int
    mqtt_rtk_status: int
    mqtt_rtcm_cnt: int
    mqtt_conn_cnt: int
    mqtt_disconn_cnt: int
    mqtt_rtk_hb_flag: int
    mqtt_rtk_hb_count: int
    mqtt_start_cnt: int
    mqtt_close_cnt: int
    mqtt_rtk_ssl_fail: int
    mqtt_rtk_wifi_config: int
    nrtk_svc_prov: int
    nrtk_svc_err: int
    base_stn_id: int
    rtk_status: int
    charge_status: int
    chassis_state: int
    nav: str
    ins_fusion: str
    perception: str
    vision_proxy: str
    vslam_vio: str
    iot_con_timeout: int
    iot_con: int
    iot_con_fail_max: str
    iot_con_fail_min: Annotated[str, Alias("iot_con__fail_min")]
    iot_url_count: int
    iot_url_max: str
    iot_url_min: str
    iot_cn: int
    iot_ap: int
    iot_us: int
    iot_eu: int
    task_area: float
    task_count: int
    task_hash: str
    systemio_boot_time: Annotated[str, Alias("systemioBootTime")]
    dds_no_gdc: int

    class Config(BaseConfig):
        allow_deserialization_not_by_alias = True


@dataclass
class CheckData(DataClassORJSONMixin):
    result: str
    error: Annotated[list[int], Alias("Error")]
    warn: Annotated[list[int], Alias("Warn")]
    ok: Annotated[list[int], Alias("OK")]

    class Config(BaseConfig):
        allow_deserialization_not_by_alias = True


@dataclass
class DeviceProperties(DataClassORJSONMixin):
    device_state: Annotated[int, Alias("deviceState")]
    battery_percentage: Annotated[int, Alias("batteryPercentage")]
    device_version: Annotated[str, Alias("deviceVersion")]
    knife_height: Annotated[int, Alias("knifeHeight")]
    lora_general_config: Annotated[str, Alias("loraGeneralConfig")]
    ext_mod: Annotated[str, Alias("extMod")]
    int_mod: Annotated[str, Alias("intMod")]
    iot_state: Annotated[int, Alias("iotState")]
    iot_msg_total: Annotated[int, Alias("iotMsgTotal")]
    iot_msg_hz: Annotated[int, Alias("iotMsgHz")]
    lt_mr_mod: Annotated[str, Alias("ltMrMod")]
    rt_mr_mod: Annotated[str, Alias("rtMrMod")]
    bms_hardware_version: Annotated[str, Alias("bmsHardwareVersion")]
    stm32_h7_version: Annotated[str, Alias("stm32H7Version")]
    left_motor_version: Annotated[str, Alias("leftMotorVersion")]
    right_motor_version: Annotated[str, Alias("rightMotorVersion")]
    rtk_version: Annotated[str, Alias("rtkVersion")]
    bms_version: Annotated[str, Alias("bmsVersion")]
    mc_boot_version: Annotated[str, Alias("mcBootVersion")]
    left_motor_boot_version: Annotated[str, Alias("leftMotorBootVersion")]
    right_motor_boot_version: Annotated[str, Alias("rightMotorBootVersion")]

    # Nested JSON objects
    device_version_info: Annotated[DeviceVersionInfo, Alias("deviceVersionInfo")]
    coordinate: Coordinate
    device_other_info: Annotated[DeviceOtherInfo, Alias("deviceOtherInfo")]
    network_info: Annotated[NetworkInfo, Alias("networkInfo")]
    check_data: Annotated[CheckData, Alias("checkData")]
    iot_id: str = ""

    class Config(BaseConfig):
        allow_deserialization_not_by_alias = True
        # Custom deserializer for nested JSON strings
        serialization_strategy = {
            DeviceVersionInfo: {
                "deserialize": lambda x: DeviceVersionInfo.from_json(x) if isinstance(x, str) else x,
                "serialize": lambda x: x.to_json() if hasattr(x, "to_json") else x,
            },
            Coordinate: {
                "deserialize": lambda x: Coordinate.from_json(x) if isinstance(x, str) else x,
                "serialize": lambda x: x.to_json() if hasattr(x, "to_json") else x,
            },
            DeviceOtherInfo: {
                "deserialize": lambda x: DeviceOtherInfo.from_json(x) if isinstance(x, str) else x,
                "serialize": lambda x: x.to_json() if hasattr(x, "to_json") else x,
            },
            NetworkInfo: {
                "deserialize": lambda x: NetworkInfo.from_json(x) if isinstance(x, str) else x,
                "serialize": lambda x: x.to_json() if hasattr(x, "to_json") else x,
            },
            CheckData: {
                "deserialize": lambda x: CheckData.from_json(x) if isinstance(x, str) else x,
                "serialize": lambda x: x.to_json() if hasattr(x, "to_json") else x,
            },
        }



from dataclasses import dataclass
from typing import Annotated, Any, Literal, Union

from mashumaro import DataClassDictMixin
from mashumaro.config import BaseConfig
from mashumaro.mixins.orjson import DataClassORJSONMixin
from mashumaro.types import Alias

from pymammotion.data.mqtt.mammotion_properties import DeviceProperties


@dataclass
class BatteryPercentageItems(DataClassORJSONMixin):
    batteryPercentage: int


@dataclass
class BMSHardwareVersionItems(DataClassORJSONMixin):
    bmsHardwareVersion: str


@dataclass
class CoordinateItems(DataClassORJSONMixin):
    coordinate: str  # '{"lon":0.303903,"lat":1.051868}'


@dataclass
class DeviceStateItems(DataClassORJSONMixin):
    deviceState: int


@dataclass
class DeviceVersionItems(DataClassORJSONMixin):
    deviceVersion: str


@dataclass
class DeviceVersionInfoItems(DataClassORJSONMixin):
    deviceVersionInfo: str


@dataclass
class ESP32VersionItems(DataClassORJSONMixin):
    esp32Version: str


@dataclass
class LeftMotorBootVersionItems(DataClassORJSONMixin):
    leftMotorBootVersion: str


@dataclass
class LeftMotorVersionItems(DataClassORJSONMixin):
    leftMotorVersion: str


@dataclass
class MCBootVersionItems(DataClassORJSONMixin):
    mcBootVersion: str


@dataclass
class NetworkInfoItems(DataClassORJSONMixin):
    networkInfo: str


@dataclass
class RightMotorBootVersionItems(DataClassORJSONMixin):
    rightMotorBootVersion: str


@dataclass
class RightMotorVersionItems(DataClassORJSONMixin):
    rightMotorVersion: str


@dataclass
class RTKVersionItems(DataClassORJSONMixin):
    rtkVersion: str


@dataclass
class StationRTKVersionItems(DataClassORJSONMixin):
    stationRtkVersion: str


@dataclass
class STM32H7VersionItems(DataClassORJSONMixin):
    stm32H7Version: str


@dataclass
class OTAProgressItems(DataClassORJSONMixin):
    result: int
    otaId: str
    progress: int
    message: str
    version: str
    properties: str


ItemTypes = Union[
    BatteryPercentageItems,
    BMSHardwareVersionItems,
    CoordinateItems,
    DeviceStateItems,
    DeviceVersionItems,
    DeviceVersionInfoItems,
    ESP32VersionItems,
    LeftMotorBootVersionItems,
    LeftMotorVersionItems,
    MCBootVersionItems,
    NetworkInfoItems,
    RightMotorBootVersionItems,
    RightMotorVersionItems,
    RTKVersionItems,
    StationRTKVersionItems,
    STM32H7VersionItems,
    OTAProgressItems,
]


@dataclass
class Item(DataClassDictMixin):
    time: int
    value: int | float | str | dict[str, Any] | ItemTypes  # Depending on the type of value


@dataclass
class Items(DataClassDictMixin):
    iotState: Item | None = None
    extMod: Item | None = None
    deviceVersionInfo: Item | None = None
    leftMotorBootVersion: Item | None = None
    knifeHeight: Item | None = None
    rtMrMod: Item | None = None
    iotMsgHz: Item | None = None
    iotMsgTotal: Item | None = None
    loraRawConfig: Item | None = None
    loraGeneralConfig: Item | None = None
    leftMotorVersion: Item | None = None
    intMod: Item | None = None
    coordinate: Item | None = None
    bmsVersion: Item | None = None
    rightMotorVersion: Item | None = None
    stm32H7Version: Item | None = None
    rightMotorBootVersion: Item | None = None
    deviceVersion: Item | None = None
    rtkVersion: Item | None = None
    ltMrMod: Item | None = None
    networkInfo: Item | None = None
    bmsHardwareVersion: Item | None = None
    batteryPercentage: Item | None = None
    deviceState: Item | None = None
    deviceOtherInfo: Item | None = None
    mcBootVersion: Item | None = None
    otaProgress: Item | None = None


@dataclass
class Params(DataClassORJSONMixin):
    device_type: Annotated[Literal["LawnMower", "Tracker"], Alias("deviceType")]
    check_failed_data: Annotated[dict[str, Any], Alias("checkFailedData")]
    group_id_list: Annotated[list[str], Alias("groupIdList")]
    _tenant_id: Annotated[str, Alias("_tenantId")]
    group_id: Annotated[str, Alias("groupId")]
    category_key: Annotated[Literal["LawnMower", "Tracker"], Alias("categoryKey")]
    batch_id: Annotated[str, Alias("batchId")]
    gmt_create: Annotated[int, Alias("gmtCreate")]
    product_key: Annotated[str, Alias("productKey")]
    generate_time: Annotated[int, Alias("generateTime")]
    device_name: Annotated[str, Alias("deviceName")]
    _trace_id: Annotated[str, Alias("_traceId")]
    iot_id: Annotated[str, Alias("iotId")]
    jmsx_delivery_count: Annotated[int, Alias("JMSXDeliveryCount")]
    check_level: Annotated[int, Alias("checkLevel")]
    qos: int
    request_id: Annotated[str, Alias("requestId")]
    _category_key: Annotated[str, Alias("_categoryKey")]
    namespace: str
    tenant_id: Annotated[str, Alias("tenantId")]
    thing_type: Annotated[Literal["DEVICE"], Alias("thingType")]
    items: Annotated["Items", Alias("items")]
    tenant_instance_id: Annotated[str, Alias("tenantInstanceId")]

    class Config(BaseConfig):
        allow_deserialization_not_by_alias = True


@dataclass
class ThingPropertiesMessage(DataClassORJSONMixin):
    method: Literal["thing.properties"]
    id: str
    params: Params
    version: Literal["1.0"]


@dataclass
class MammotionPropertiesMessage(DataClassORJSONMixin):
    id: str
    version: str
    sys: dict
    params: DeviceProperties



# === sendOrderMsg_Media ===
from abc import ABC

from pymammotion.mammotion.commands.abstract_message import AbstractMessage
from pymammotion.proto import (
    GetHeadlamp,
    LampCtrlSta,
    LampManualCtrlSta,
    LubaMsg,
    MsgAttr,
    MsgCmdType,
    MsgDevice,
    MulLanguage,
    MulSetAudio,
    MulSetWiper,
    MulSex,
    SetHeadlamp,
    SocMul,
)


class MessageMedia(AbstractMessage, ABC):
    def send_order_msg_media(self, mul):
        luba_msg = LubaMsg(
            msgtype=MsgCmdType.MUL,
            sender=MsgDevice.DEV_MOBILEAPP,
            rcver=self.get_msg_device(MsgCmdType.MUL, MsgDevice.SOC_MODULE_MULTIMEDIA),
            msgattr=MsgAttr.REQ,
            seqs=self.seqs.increment_and_get() & 255,
            version=1,
            subtype=self.user_account,
            mul=mul,
        )

        return luba_msg.SerializeToString()

    def set_car_volume(self, volume: int):
        """Set the car volume. 0 - 100"""
        return self.send_order_msg_media(SocMul(set_audio=MulSetAudio(at_switch=volume)))

    def set_car_voice_language(self, language_type: MulLanguage | str | None):
        return self.send_order_msg_media(SocMul(set_audio=MulSetAudio(au_language=language_type)))

    def set_car_volume_sex(self, sex: MulSex):
        return self.send_order_msg_media(SocMul(set_audio=MulSetAudio(sex=sex)))

    def set_car_wiper(self, round_num: int):
        """Set mower wiper."""
        # 2
        return self.send_order_msg_media(SocMul(set_wiper=MulSetWiper(round=round_num)))

    def get_car_light(self, ids: int):
        """Get mower light settings.
        1126 for manual
        1123 for night time settings
        """
        return self.send_order_msg_media(SocMul(get_lamp=GetHeadlamp(get_ids=ids)))

    def set_car_light(self, on_off: bool = False):
        """Set mower light.

        set whether light is on during the night during mowing
        auto night on true, id=1121, power_ctrl=1
        auto night off false, id=1121, power_ctrl=1
        """

        ctrl_state = LampCtrlSta.power_ctrl_on if on_off else LampCtrlSta.power_off
        return self.send_order_msg_media(
            SocMul(
                set_lamp=SetHeadlamp(
                    set_ids=1121, lamp_power_ctrl=1, lamp_ctrl=ctrl_state, ctrl_lamp_bright=False, lamp_bright=0
                )
            )
        )

    def set_car_manual_light(self, manual_ctrl: bool = False):
        """Set mower light.

        set whether light is on manually
        manual on: true, id=1125, power_ctrl=2
        manual off: false, id=1127, power_ctrl=2
        """
        ids = 1125 if manual_ctrl else 1127
        manual_light_ctrl = LampManualCtrlSta.manual_power_on if manual_ctrl else LampManualCtrlSta.manual_power_off
        return self.send_order_msg_media(
            SocMul(set_lamp=SetHeadlamp(set_ids=ids, lamp_power_ctrl=2, lamp_manual_ctrl=manual_light_ctrl))
        )



"""Mower device with cloud MQTT connectivity."""

import logging

from pymammotion.aliyun.model.dev_by_account_response import Device
from pymammotion.data.mower_state_manager import MowerStateManager
from pymammotion.mammotion.devices.mammotion_cloud import MammotionBaseCloudDevice, MammotionCloud
from pymammotion.mammotion.devices.mower_device import MammotionMowerDevice

_LOGGER = logging.getLogger(__name__)


class MammotionMowerCloudDevice(MammotionBaseCloudDevice, MammotionMowerDevice):
    """Mower device with cloud connectivity and map synchronization."""

    def __init__(self, mqtt: MammotionCloud, cloud_device: Device, state_manager: MowerStateManager) -> None:
        """Initialize MammotionMowerCloudDevice.

        Uses multiple inheritance to combine:
        - MammotionBaseCloudDevice: MQTT communication
        - MammotionMowerDevice: Map sync callbacks
        """
        # Initialize base cloud device (which also initializes MammotionBaseDevice)
        super().__init__(mqtt, cloud_device, state_manager)
        # Initialize mower device callbacks (but skip base device init as it's already done)
        # We manually set the callbacks that MammotionMowerDevice would set
        self._state_manager.cloud_gethash_ack_callback = self.datahash_response
        self._state_manager.cloud_get_commondata_ack_callback = self.commdata_response
        self._state_manager.cloud_get_plan_callback = self.plan_callback

    def __del__(self) -> None:
        """Cleanup subscriptions and callbacks."""
        # Clean up mower-specific callbacks
        if hasattr(self, "_state_manager"):
            self._state_manager.cloud_gethash_ack_callback = None
            self._state_manager.cloud_get_commondata_ack_callback = None
            self._state_manager.cloud_get_plan_callback = None
        # Call parent cleanup
        super().__del__()



from dataclasses import dataclass, field
from enum import StrEnum

from mashumaro.mixins.orjson import DataClassORJSONMixin


class NetUsedType(StrEnum):
    NONE = "NONE"
    WIFI = "WIFI"
    MNET = "MNET"


@dataclass
class ConnectData(DataClassORJSONMixin):
    connect_type: int = 0
    ble_rssi: int = 0
    wifi_rssi: int = 0
    link_type: int = 0
    mnet_rssi: int = 0
    mnet_inet: int = 0
    used_net: str = "NONE"


#    mnet_cfg:


@dataclass
class CollectorStatus(DataClassORJSONMixin):
    collector_installation_status: int = 0


@dataclass
class MnetInfo(DataClassORJSONMixin):
    model: str = ""
    revision: str = ""
    imei: str = ""


@dataclass
class LockStateT(DataClassORJSONMixin):
    lock_state: int = 0


@dataclass
class VioSurvivalInfo(DataClassORJSONMixin):
    vio_survival_distance: float = 0.0


@dataclass
class FpvInfo(DataClassORJSONMixin):
    """fpv_flag: 0: no fpv, 1: fpv ok, 2: fpv error"""

    fpv_flag: int = 0
    wifi_available: int = 0
    mobile_net_available: int = 0


@dataclass
class DeviceData(DataClassORJSONMixin):
    sys_status: int = 0
    charge_state: int = 0
    battery_val: int = 0
    sensor_status: int = 0
    last_status: int = 0
    sys_time_stamp: str = ""
    vslam_status: int = 0
    mnet_info: MnetInfo = field(default_factory=MnetInfo)
    vio_survival_info: VioSurvivalInfo = field(default_factory=VioSurvivalInfo)
    collector_status: CollectorStatus = field(default_factory=CollectorStatus)
    fpv_info: FpvInfo | None = None
    lock_state: LockStateT = field(default_factory=LockStateT)


@dataclass
class RTKDisStatus(DataClassORJSONMixin):
    pos_status: int = 0
    precision: int = 0
    device_signal: int = 0
    l1: int = 0
    l2: int = 0
    connection_to_ref: int = 0
    rtk_signal: int = 0


@dataclass
class RTKData(DataClassORJSONMixin):
    status: int = 0
    pos_level: int = 0
    gps_stars: int = 0
    dis_status: int = 0
    co_view_stars: int = 0

    def get_dis_status(self) -> RTKDisStatus:
        rtk_dis_status = RTKDisStatus()
        rtk_dis_status.pos_status = ((int)(self.dis_status >> 8)) & 255
        rtk_dis_status.precision = ((int)(self.dis_status >> 56)) & 255
        rtk_dis_status.device_signal = ((int)(self.dis_status >> 32)) & 255
        rtk_dis_status.l1 = ((int)(self.dis_status >> 16)) & 255
        rtk_dis_status.l2 = ((int)(self.dis_status >> 24)) & 255
        rtk_dis_status.connection_to_ref = ((int)(self.dis_status >> 48)) & 255
        rtk_dis_status.rtk_signal = ((int)(self.dis_status >> 40)) & 255
        return rtk_dis_status


@dataclass
class LocationData(DataClassORJSONMixin):
    real_pos_x: int = 0
    real_pos_y: int = 0
    real_toward: int = 0
    pos_type: int = 0
    bol_hash: int = 0


@dataclass
class BladeUsed(DataClassORJSONMixin):
    blade_used_time: int = 0
    blade_used_warn_time: int = 0


@dataclass
class Maintain(DataClassORJSONMixin):
    blade_used_time: BladeUsed = field(default_factory=BladeUsed)
    mileage: int = 0
    work_time: int = 0
    bat_cycles: int = 0


@dataclass
class VisionInfo(DataClassORJSONMixin):
    x: float = 0.0
    y: float = 0.0
    heading: float = 0.0
    vio_state: int = 0
    brightness: int = 0
    detect_feature_num: int = 0
    track_feature_num: int = 0


@dataclass
class HeadingState(DataClassORJSONMixin):
    heading_state: int = 0


@dataclass
class WorkData(DataClassORJSONMixin):
    path: int = 0
    path_hash: int = 0
    progress: int = 0
    area: int = 0
    bp_info: int = 0
    bp_hash: int = 0
    bp_pos_x: int = 0
    bp_pos_y: int = 0
    real_path_num: int = 0
    path_pos_x: int = 0
    path_pos_y: int = 0
    ub_zone_hash: int = 0
    ub_path_hash: int = 0
    init_cfg_hash: int = 0
    ub_ecode_hash: int = 0
    nav_run_mode: int = 0
    test_mode_status: int = 0
    man_run_speed: int = 0
    nav_edit_status: int = 0
    knife_height: int = 0
    nav_heading_state: HeadingState = field(default_factory=HeadingState)
    cutter_offset: float = 0.0
    cutter_width: float = 0.0


@dataclass
class ReportData(DataClassORJSONMixin):
    connect: ConnectData = field(default_factory=ConnectData)
    dev: DeviceData = field(default_factory=DeviceData)
    maintenance: Maintain = field(default_factory=Maintain)
    vision_info: VisionInfo = field(default_factory=VisionInfo)
    rtk: RTKData = field(default_factory=RTKData)
    locations: list[LocationData] = field(default_factory=list)
    work: WorkData = field(default_factory=WorkData)

    def update(self, data: dict) -> None:
        locations = self.locations
        if data.get("locations") is not None:
            locations = [LocationData.from_dict(loc) for loc in data.get("locations", [])]

        self.connect = ConnectData.from_dict(data.get("connect", self.connect.to_dict()))
        self.dev = DeviceData.from_dict(data.get("dev", self.dev.to_dict()))
        self.rtk = RTKData.from_dict(data.get("rtk", self.rtk.to_dict()))
        self.maintenance = Maintain.from_dict(data.get("maintain", self.maintenance.to_dict()))
        self.vision_info = VisionInfo.from_dict(data.get("vio_to_app_info", VisionInfo().to_dict()))
        self.locations = locations
        self.work = WorkData.from_dict(data.get("work", self.work.to_dict()))



# === sendOrderMsg_Ota ===
from abc import ABC
import logging

from pymammotion.mammotion.commands.abstract_message import AbstractMessage
from pymammotion.proto import GetInfoReq, InfoType, LubaMsg, MctlOta, MsgAttr, MsgCmdType, MsgDevice

_LOGGER = logging.getLogger(__name__)


class MessageOta(AbstractMessage, ABC):
    """Message OTA class."""

    def send_order_msg_ota(self, ota):
        luba_msg = LubaMsg(
            msgtype=MsgCmdType.EMBED_OTA,
            sender=MsgDevice.DEV_MOBILEAPP,
            rcver=self.get_msg_device(MsgCmdType.EMBED_OTA, MsgDevice.DEV_MAINCTL),
            msgattr=MsgAttr.REQ,
            seqs=self.seqs.increment_and_get() & 255,
            version=1,
            subtype=self.user_account,
            ota=ota,
        )

        return luba_msg.SerializeToString()

    def get_device_ota_info(self, log_type: int):
        todev_get_info_req = MctlOta(todev_get_info_req=GetInfoReq(type=InfoType.IT_OTA))

        _LOGGER.debug("===Send command to get upgrade details===logType:" + str(log_type))
        return self.send_order_msg_ota(todev_get_info_req)

    def get_device_info_new(self) -> bytes:
        """New device call for OTA upgrade information."""
        todev_get_info_req = MctlOta(todev_get_info_req=GetInfoReq(type=InfoType.IT_BASE))
        _LOGGER.debug("Send to get OTA upgrade information", "Get device information")
        return self.send_order_msg_ota(todev_get_info_req)
